/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.client;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMultipart;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.message.BasicHeader;
import org.apache.log4j.Logger;

/**
 * Connects the Java Mail MimeMultipart to Apache Commons file-upload
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public final class MimeMultipartEntity implements HttpEntity {
    private static final String HEADER_CONTENT_ENCODING = "Content-Encoding";
    private static final String HEADER_CONTENT_TYPE = "Content-Type";

    private static final String encoding = "US-ASCII";

    private final Logger log = Logger.getLogger(MimeMultipartEntity.class);
    private final MimeMultipart mmp;

    public MimeMultipartEntity(final MimeMultipart mmp) {
	this.mmp = mmp;
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#consumeContent()
     */
    public void consumeContent() {
	// null operation
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#getContent()
     */
    public InputStream getContent() throws IOException, IllegalStateException {
	final PipedInputStream is = new PipedInputStream();
	final PipedOutputStream os = new PipedOutputStream(is);
	new Thread(new Runnable() {
	    public void run() {
		try {
		    mmp.writeTo(os);
		} catch (IOException e) {
		    log.error("Unable to copy MimeMultipart content", e);
		} catch (MessagingException e) {
		    log.error("Unable to copy MimeMultipart content", e);
		}
	    }
	}).start();
	return is;
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#getContentEncoding()
     */
    public Header getContentEncoding() {
	return new BasicHeader(HEADER_CONTENT_ENCODING, encoding);
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#getContentLength()
     */
    public long getContentLength() {
	return -1;	// no easy way to determine content length
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#getContentType()
     */
    public Header getContentType() {
	return new BasicHeader(HEADER_CONTENT_TYPE, mmp.getContentType());
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#isChunked()
     */
    public boolean isChunked() {
	return false;
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#isRepeatable()
     */
    public boolean isRepeatable() {
	return false;
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#isStreaming()
     */
    public boolean isStreaming() {
	return false;
    }

    /* (non-Javadoc)
     * @see org.apache.http.HttpEntity#writeTo(java.io.OutputStream)
     */
    public void writeTo(final OutputStream os) throws IOException {
	try {
	    mmp.writeTo(os);
	} catch (MessagingException e) {
	    throw new IOException("write failed: " + e.getMessage());
	}
    }

}
